@@ -174,11 +174,10 @@ func processBGPSummary(ch chan<- prometheus.Metric, jsonBGPSum []byte, AFI strin
 		return err
 	}
 
-	var peerDescJSON map[string]map[string]string
-	var peerDescText map[string]string
+	var peerDesc map[string]bgpVRF
 	var err error
 	if *bgpPeerTypes || *bgpPeerDescs {
-		peerDescJSON, peerDescText, err = getBGPPeerDesc(logger)
+		peerDesc, err = getBGPPeerDesc()
 		if err != nil {
 			return err
 		}
@@ -204,14 +203,20 @@ func processBGPSummary(ch chan<- prometheus.Metric, jsonBGPSum []byte, AFI strin
 				peerLabels := []string{strings.ToLower(vrfName), strings.ToLower(AFI), strings.ToLower(SAFI), localAs, peerIP, strconv.FormatUint(uint64(peerData.RemoteAs), 10)}
 
 				if *bgpPeerDescs {
-					d := ""
+					d := peerDesc[vrfName].BGPNeighbors[peerIP].Desc
 					if *bgpPeerDescsText {
-						d = peerDescText[peerIP]
+						// The labels are "vrf", "afi", "safi", "local_as", "peer", "remote_as", "peer_desc"
+						peerLabels = append(peerLabels, d)
 					} else {
-						d = peerDescJSON[peerIP]["desc"]
+						// Assume the FRR BGP neighbor description is JSON formatted, and the description is in the "desc" field.
+						jsonDesc := struct{ Desc string }{}
+						if err := json.Unmarshal([]byte(d), &jsonDesc); err != nil {
+							// Don't return an error as unmarshalling is best effort.
+							level.Error(logger).Log("msg", "cannot unmarshal bgp description", "description", peerDesc[vrfName].BGPNeighbors[peerIP].Desc, "err", err)
+						}
+						// The labels are "vrf", "afi", "safi", "local_as", "peer", "remote_as", "peer_desc"
+						peerLabels = append(peerLabels, jsonDesc.Desc)
 					}
-					// The labels are "vrf", "afi", "safi", "local_as", "peer", "remote_as", "peer_desc"
-					peerLabels = append(peerLabels, d)
 				}
 
 				// In earlier versions of FRR did not expose a summary of advertised prefixes for all peers, but in later versions it can get with PfxSnt field.
@@ -235,11 +240,17 @@ func processBGPSummary(ch chan<- prometheus.Metric, jsonBGPSum []byte, AFI strin
 				}
 				newGauge(ch, bgpDesc["prefixReceivedCount"], prefixReceived, peerLabels...)
 
+				var peerDescTypes map[string]string
 				if *bgpPeerTypes {
+					if err := json.Unmarshal([]byte(peerDesc[vrfName].BGPNeighbors[peerIP].Desc), &peerDescTypes); err != nil {
+						// Don't return an error as unmarshalling is best effort.
+						level.Error(logger).Log("msg", "cannot unmarshal bgp description", "description", peerDesc[vrfName].BGPNeighbors[peerIP].Desc, "err", err)
+					}
+
 					for _, descKey := range *frrBGPDescKey {
-						if peerDescJSON[peerIP][descKey] != "" {
-							if _, exist := peerTypes[strings.TrimSpace(peerDescJSON[peerIP][descKey])]; !exist {
-								peerTypes[strings.TrimSpace(peerDescJSON[peerIP][descKey])] = 0
+						if peerDescTypes[descKey] != "" {
+							if _, exist := peerTypes[strings.TrimSpace(peerDescTypes[descKey])]; !exist {
+								peerTypes[strings.TrimSpace(peerDescTypes[descKey])] = 0
 							}
 						}
 					}
@@ -250,8 +261,8 @@ func processBGPSummary(ch chan<- prometheus.Metric, jsonBGPSum []byte, AFI strin
 					peerState = 1
 					if *bgpPeerTypes {
 						for _, descKey := range *frrBGPDescKey {
-							if peerDescJSON[peerIP][descKey] != "" {
-								peerTypes[strings.TrimSpace(peerDescJSON[peerIP][descKey])]++
+							if peerDescTypes[descKey] != "" {
+								peerTypes[strings.TrimSpace(peerDescTypes[descKey])]++
 							}
 						}
 					}
@@ -315,7 +326,7 @@ type bgpPeerSession struct {
 	RemoteAs            uint32
 	MsgRcvd             uint32
 	MsgSent             uint32
-	PeerUptimeMsec      uint32
+	PeerUptimeMsec      uint64
 	PrefixReceivedCount uint32
 	PfxRcd              uint32
 	PfxSnt              *uint32
@@ -324,37 +335,61 @@ type bgpAdvertisedRoutes struct {
 	TotalPrefixCounter uint32 `json:"totalPrefixCounter"`
 }
 
-// Returns:
-//  - Map from JSON formatted BGP peer descriptions
-//  - Plain text description of peers
-//  - Error
-func getBGPPeerDesc(logger log.Logger) (map[string]map[string]string, map[string]string, error) {
-	descJSON := make(map[string]map[string]string)
-	descText := make(map[string]string)
+func getBGPPeerDesc() (map[string]bgpVRF, error) {
 
-	output, err := executeBGPCommand("show bgp neighbors json")
+	output, err := executeBGPCommand("show bgp vrf all neighbors json")
 	if err != nil {
-		return nil, nil, err
+		return nil, err
+	}
+	return processBGPPeerDesc(output)
+}
+
+func processBGPPeerDesc(output []byte) (map[string]bgpVRF, error) {
+	vrfMap := make(map[string]bgpVRF)
+	if err := json.Unmarshal([]byte(output), &vrfMap); err != nil {
+		return nil, err
+	}
+	return vrfMap, nil
+}
+
+func (vrf *bgpVRF) UnmarshalJSON(data []byte) error {
+	var raw map[string]*json.RawMessage
+
+	if err := json.Unmarshal(data, &raw); err != nil {
+		return err
 	}
 
-	var neighbors map[string]bgpBGPNeighbor
-	if err := json.Unmarshal(output, &neighbors); err != nil {
-		return nil, nil, err
-	}
-	for neighbor, data := range neighbors {
-		if !*bgpPeerDescsText {
-			var peerDesc map[string]string
-			if err := json.Unmarshal([]byte(data.NbrDesc), &peerDesc); err != nil {
-				// Don't return an error as unmarshalling is best effort.
-				level.Error(logger).Log("msg", "cannot unmarshall bgp description", "description", data.NbrDesc, "neighbor", neighbor, "err", err)
+	vrf.BGPNeighbors = make(map[string]bgpNeighbor)
+
+	for k, v := range raw {
+		switch k {
+		case "vrfId":
+			if err := json.Unmarshal(*v, &vrf.ID); err != nil {
+				return err
 			}
-			descJSON[neighbor] = peerDesc
+		case "vrfName":
+			// This is somewhat redundant, since the VRF name is a top-level key in the source JSON.
+			if err := json.Unmarshal(*v, &vrf.Name); err != nil {
+				return err
+			}
+		default:
+			var neighbor bgpNeighbor
+			if err := json.Unmarshal(*v, &neighbor); err != nil {
+				return err
+			}
+
+			vrf.BGPNeighbors[k] = neighbor
 		}
-		descText[neighbor] = data.NbrDesc
 	}
-	return descJSON, descText, nil
+	return nil
 }
 
-type bgpBGPNeighbor struct {
-	NbrDesc string `json:"nbrDesc"`
+type bgpVRF struct {
+	ID           int                    `json:"vrfId"`
+	Name         string                 `json:"vrfName"`
+	BGPNeighbors map[string]bgpNeighbor `json:"-"`
+}
+
+type bgpNeighbor struct {
+	Desc string `json:"nbrDesc"`
 }
